/**
 * 
 */
package org.processmining.plugins.log.visualization;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;

/**
 * @author aadrians Jul 28, 2012
 * 
 */
public class LogLastOccurrenceProvider extends LogPerformanceMatrixProvider {

	public LogLastOccurrenceProvider(XLog log, XEventClassifier classifier) {
		super();
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log, classifier);
		XEventClasses ec = logInfo.getEventClasses();

		// encode event class
		Collection<XEventClass> ecs = ec.getClasses();
		List<XEventClass> lst = new ArrayList<XEventClass>(ecs);
		Collections.sort(lst, new Comparator<XEventClass>() {

			public int compare(XEventClass o1, XEventClass o2) {
				return o1.getId().compareTo(o2.getId());
			}
		});
		
		XEventClass[] int2class = new XEventClass[lst.size()];
		Iterator<XEventClass> it = lst.iterator();
		for (int i =0 ; i < int2class.length; i++){
			int2class[i] = it.next();
		}
		
		Map<XEventClass, Integer> class2int = new HashMap<XEventClass, Integer>(int2class.length);
		classLbl = new String[int2class.length];
		for (int i = 0; i < int2class.length; i++) {
			class2int.put(int2class[i], i);
			classLbl[i] = int2class[i].getId();
		}

		// variables to measure averages and standard deviations
		int blockSize = int2class.length * int2class.length;
		data = new double[blockSize * 4];
		Arrays.fill(data, Double.NaN);
		occ = new int[blockSize]; // number of occurrence
		Arrays.fill(occ, 0);

		// calculate elapsed time between transitions
		long[] temp = new long[int2class.length];

		XTimeExtension tx = XTimeExtension.instance();
		for (XTrace t : log) {
			Arrays.fill(temp, Long.MIN_VALUE);
			for (XEvent evt : t) {
				// check if the transition has been recorded for this batch
				int idx = class2int.get(ec.getClassOf(evt));
				Date timestamp = tx.extractTimestamp(evt);
				if (timestamp != null) {
					temp[idx] = timestamp.getTime();
				}
			}

			// move the calculation to the main arrays
			for (int i = 0; i < temp.length; i++) {
				if (temp[i] > Long.MIN_VALUE) {
					for (int j = i + 1; j < temp.length; j++) {
						if (temp[j] > Long.MIN_VALUE) {
							if (temp[i] < temp[j]) {
								updateMandSVal(i, j, data, occ, temp[j] - temp[i], int2class.length);
							} else if (temp[i] > temp[j]) {
								updateMandSVal(j, i, data, occ, temp[i] - temp[j], int2class.length);
							} else if (temp[i] == temp[j]) {
								// at the same time
								updateMandSVal(j, i, data, occ, 0, int2class.length);
								updateMandSVal(i, j, data, occ, 0, int2class.length);
							}
						}
					}
				}
			}
		}

		// finalize calculation of average and stddev
		for (int i = 0; i < blockSize; i++) {
			int loc = 4 * i;
			data[loc + 3] = Math.sqrt(data[loc + 3] / (occ[i] - 1));
		}
	}
}
