/**
 * 
 */
package org.processmining.plugins.log.visualization;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.plugins.log.visualization.LogPerformanceMatrixProvider.Metric;
import org.processmining.plugins.performancematrix.PerformanceMatrixPanel;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

/**
 * @author aadrians Jul 26, 2012
 * 
 */
@Plugin(name = "Synchronous activity analysis", returnLabels = { "Performance Matrix" }, returnTypes = { JComponent.class }, parameterLabels = { "Log" }, userAccessible = false)
@Visualizer
public class LogPerformanceMatrix {
	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context, XLog log) {
		Set<XEventClassifier> classifiersSet = new HashSet<XEventClassifier>(log.getClassifiers());
		classifiersSet.add(XLogInfoImpl.STANDARD_CLASSIFIER);
		classifiersSet.add(XLogInfoImpl.NAME_CLASSIFIER);
		classifiersSet.add(XLogInfoImpl.RESOURCE_CLASSIFIER);
		Object[] classifiers = classifiersSet.toArray();
		
		// HV: This is bad: in a headless context, we start popping up dialogs. FIXME FIXME FIXME
		XEventClassifier classifier = (XEventClassifier) JOptionPane.showInputDialog(new JPanel(),
				"Choose event classifier", "Classifier Selection", JOptionPane.PLAIN_MESSAGE, null, classifiers,
				XLogInfoImpl.STANDARD_CLASSIFIER);
		if (classifier == null) {
			return new JLabel("Synchronous activity analysis canceled");
		}
		String selection = (String) JOptionPane.showInputDialog(new JPanel(),
				"Which occurrence of events are considered", "Preserved events", JOptionPane.PLAIN_MESSAGE, null, new Object[] {"First occurrences", "Last occurrences"},
				"First occurrrences");
		if (selection == null) {
			return new JLabel("Synchronous activity analysis canceled");
		}
		return createPerformanceMatrix(log, classifier, selection.equals("First occurrrences"));
	}

	/**
	 * @param logReplayResult
	 * @param net
	 * @param log
	 * @param initMarking
	 * @return
	 */
	public static JComponent createPerformanceMatrix(XLog log, XEventClassifier classifier, boolean isFirst) {
		final LogPerformanceMatrixProvider provider;
		
		if (isFirst){
			provider = new LogFirstOccurrenceProvider(log, classifier);
		} else {
			provider = new LogLastOccurrenceProvider(log, classifier);
		}

		final double[] data = new double[provider.getNumClasses() * provider.getNumClasses()];
		Arrays.fill(data, Double.NaN);

		int dataPtr = 0;
		for (int i = 0; i < provider.getNumClasses(); i++) {
			for (int j = 0; j < provider.getNumClasses(); j++) {
				data[dataPtr] = provider.getTimeBetween(i, j, LogFirstOccurrenceProvider.Metric.AVG);
				dataPtr++;
			}
		}

		// create matrix canvas and wrap it with scrollpane
		final PerformanceMatrixPanel matrixPanel = new PerformanceMatrixPanel(provider.getClassesLabel(),
				provider.getClassesLabel(), data);
		JScrollPane scroller = new JScrollPane(matrixPanel);
		final JScrollBar vBar = scroller.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
				WidgetColors.COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(true);
		vBar.setBackground(WidgetColors.COLOR_ENCLOSURE_BG);
		final JScrollBar hBar = scroller.getHorizontalScrollBar();
		hBar.setUI(new SlickerScrollBarUI(hBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
				WidgetColors.COLOR_NON_FOCUS, 4, 12));
		hBar.setOpaque(true);
		hBar.setBackground(WidgetColors.COLOR_ENCLOSURE_BG);

		// create statistic panel
		SlickerFactory factory = SlickerFactory.instance();

		final ProMTextField threshold = new ProMTextField("0.01");
		threshold.setPreferredSize(new Dimension(100,30));

		final JComboBox changeData = factory.createComboBox(new Object[] { "Minimum Value", "Maximum Value", "Average",
				"Standard Deviation", "Frequency" });
		changeData.setPreferredSize(new Dimension(300, 30));
		changeData.setSelectedIndex(2);
		changeData.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				Metric m = null;
				int selectedIdx = changeData.getSelectedIndex();
				if (selectedIdx == 4) {
					// create data
					int ptr = 0;
					for (int i = 0; i < provider.getNumClasses(); i++) {
						for (int j = 0; j < provider.getNumClasses(); j++) {
							data[ptr] = provider.getFrequency(i, j);
							if (Double.compare(data[ptr], 0.0) == 0) {
								data[ptr] = Double.NaN;
							}
							ptr++;
						}
					}
					matrixPanel.setData(data, false, Double.parseDouble(threshold.getText()));
				} else {
					if (selectedIdx == 0) {
						m = Metric.MIN;
					} else if (selectedIdx == 1) {
						m = Metric.MAX;
					} else if (selectedIdx == 2) {
						m = Metric.AVG;
					} else if (selectedIdx == 3) {
						m = Metric.STDDEV;
					}
					// create data
					int ptr = 0;
					for (int i = 0; i < provider.getNumClasses(); i++) {
						for (int j = 0; j < provider.getNumClasses(); j++) {
							data[ptr] = provider.getTimeBetween(i, j, m);
							ptr++;
						}
					}
					matrixPanel.setData(data);
				}
				// set matrix panel
				matrixPanel.repaint();

			}
		});

		JButton updateThreshold = factory.createButton("Update");
		updateThreshold.setPreferredSize(new Dimension(100,30));
		updateThreshold.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					double t = Double.parseDouble(threshold.getText());
					if (Double.compare(t, 0.00) >= 0){
						matrixPanel.setThreshold(t);
						matrixPanel.repaint();
					}
				} catch (Exception exc){
					exc.printStackTrace();
				}
			}
		});

		JPanel statisticPanel = new JPanel();
		statisticPanel.add(factory.createLabel("Choose coloring criteria"));
		statisticPanel.add(changeData);

		statisticPanel.add(factory.createLabel("Similarity threshold"));
		statisticPanel.add(threshold);
		statisticPanel.add(updateThreshold);
		
		// MAIN PANEL
		ProMSplitPane splitPanel = new ProMSplitPane(ProMSplitPane.VERTICAL_SPLIT);
		splitPanel.setBorder(BorderFactory.createEmptyBorder());
		splitPanel.setOneTouchExpandable(true);
		splitPanel.setLeftComponent(statisticPanel);
		splitPanel.setRightComponent(scroller);

		return splitPanel;
	}
}
