/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.conformance;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;

/**
 * @author aadrians Jan 9, 2012
 * 
 */
@Plugin(name = "Verify final marking is reached by all alignments", returnLabels = { "Verification result" }, returnTypes = { String.class }, parameterLabels = {
		"Alignment", "Petrinet", "Initial Marking", "Final marking" }, help = "Verify whether alignment projected to model always reach final marking.", userAccessible = true)
public class ReachFinalMarkingVerification {
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Arya Adriansyah", email = "a.adriansyah@tue.nl", pack = "Replayer")
	@PluginVariant(variantLabel = "Given alignment and net", requiredParameterLabels = { 0, 1, 2, 3 })
	public String verifyFinalMarkingReachable(final UIPluginContext context, PNRepResult alignment, PetrinetGraph net,
			Marking initMarking, Marking finalMarking) {
		Marking[] finalMarkings = new Marking[] { finalMarking };
		return String.valueOf(verifyFinalMarkingReachability(alignment, net, initMarking, finalMarkings));
	}

	@PluginVariant(variantLabel = "Given only alignment and net", requiredParameterLabels = { 0, 1 })
	public String verifyFinalMarkingReachable(final UIPluginContext context, PNRepResult alignment, PetrinetGraph net) {
		// retrieve mapping between process model to log
		try {
			ConnectionManager connManager = context.getConnectionManager();
			// get marking
			InitialMarkingConnection initMarkingConn = connManager.getFirstConnection(InitialMarkingConnection.class,
					context, net);
			Marking initMarking = initMarkingConn.getObjectWithRole(InitialMarkingConnection.MARKING);

			Collection<FinalMarkingConnection> conns = connManager.getConnections(FinalMarkingConnection.class,
					context, net);
			Marking[] finalMarkings = new Marking[conns.size()];
			Iterator<FinalMarkingConnection> it = conns.iterator();
			for (int i = 0; i < finalMarkings.length; i++) {
				finalMarkings[i] = it.next().getObjectWithRole(FinalMarkingConnection.MARKING);
			}

			return String.valueOf(verifyFinalMarkingReachability(alignment, net, initMarking, finalMarkings));
		} catch (ConnectionCannotBeObtained noConnection) {
			// do nothing
		}
		context.getFutureResult(0).cancel(true);
		return null;
	}

	/**
	 * main method to check final marking is reachable by firing only model
	 * moves
	 * 
	 * @param alignment
	 * @param net
	 * @param initMarking
	 * @param finalMarkings
	 * @return
	 */
	public boolean verifyFinalMarkingReachability(PNRepResult alignment, PetrinetGraph net, Marking initMarking,
			Marking[] finalMarkings) {
		loop: for (SyncReplayResult res : alignment) {
			if (res.isReliable()) {
				List<Object> lst = res.getNodeInstance();

				// fire all moves on model
				Marking m = new Marking(initMarking);
				for (Object ni : lst) {
					if (ni instanceof Transition) {
						fireTransition((Transition) ni, m, net);
					}
				}

				// check equality with final marking
				for (int i = 0; i < finalMarkings.length; i++) {
					if (finalMarkings[i].equals(m)) {
						continue loop;
					}
				}
				return false;
			}
		}
		return true;
	}

	private static void fireTransition(Transition t, Marking m, PetrinetGraph net) {
		Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges = net.getInEdges(t);
		if (edges != null) {
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : edges) {
				m.add((Place) edge.getSource(), -1 * net.getArc(edge.getSource(), t).getWeight());
			}
		}
		edges = net.getOutEdges(t);
		if (edges != null) {
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : edges) {
				m.add((Place) edge.getTarget(), net.getArc(t, edge.getTarget()).getWeight());
			}
		}
	}

}
