/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.export;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Set;

import javax.swing.JOptionPane;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.EvClassLogPetrinetConnection;
import org.processmining.models.connections.petrinets.PNRepResultAllRequiredParamConnection;
import org.processmining.models.connections.petrinets.PNRepResultConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;

/**
 * Construct log based on alignment move on log steps will be just inserted as
 * it is, move on model is inserted as an event with the existing event class
 * (if the transition is mapped) or as a new
 * 
 * @author aadrians Jan 17, 2012
 * 
 */
@Plugin(name = "Construct Log from Alignment", returnLabels = { "Log" }, returnTypes = { XLog.class }, parameterLabels = {
		"Log-Model alignment", "Original Log", "Log-Net mapping" }, userAccessible = false)
public class LogCreator {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Arya Adriansyah", email = "a.adriansyah@tue.nl", pack = "PNetAlignmentAnalysis")
	@PluginVariant(requiredParameterLabels = { 0 })
	public XLog extractLog(final UIPluginContext context, PNRepResult logReplayResult) {
		TransEvClassMapping mapping = null;
		XLog log = null;

		// get the mapping and the original log
		try {
			ConnectionManager connManager = context.getConnectionManager();
			PNRepResultAllRequiredParamConnection conn = connManager.getFirstConnection(
					PNRepResultAllRequiredParamConnection.class, context, logReplayResult);
			if (conn != null) {
				PetrinetGraph pn = conn.getObjectWithRole(PNRepResultConnection.PN);
				log = conn.getObjectWithRole(PNRepResultConnection.LOG);

				// find connection to mapping between log and model
				EvClassLogPetrinetConnection conn2Log = connManager.getFirstConnection(
						EvClassLogPetrinetConnection.class, context, pn, log);
				mapping = conn2Log.getObjectWithRole(EvClassLogPetrinetConnection.TRANS2EVCLASSMAPPING);
			}
		} catch (Exception exc) {
			// no mapping can be obtained
			context.getFutureResult(0).cancel(true);
			context.log("no mapping can be obtained between Petri net and log that produce the replay result");
			return null;
		}

		// show parameter config
		// no algorithm yet, populate from context
		Set<Class<?>> coverageEstimatorClasses = context.getPluginManager().getKnownClassesAnnotatedWith(
				LogCreatorFromAlignment.class);
		List<ILogCreatorAlgorithm> algList = new ArrayList<ILogCreatorAlgorithm>(2);
		if (coverageEstimatorClasses != null) {
			for (Class<?> coverClass : coverageEstimatorClasses) {
				try {
					ILogCreatorAlgorithm alg = (ILogCreatorAlgorithm) coverClass.newInstance();
					algList.add(alg);

				} catch (InstantiationException e1) {
					// do nothing
				} catch (IllegalAccessException e1) {
					// do nothing
				} catch (Exception exc) {
					// do nothing
				}
			}
			Collections.sort(algList, new Comparator<ILogCreatorAlgorithm>() {

				public int compare(ILogCreatorAlgorithm o1, ILogCreatorAlgorithm o2) {
					return o1.toString().compareTo(o2.toString());
				}
			});
		}
		ILogCreatorAlgorithm[] arr = algList.toArray(new ILogCreatorAlgorithm[algList.size()]);
		LogCreatorPanel panel = new LogCreatorPanel(arr);
		if (context.showConfiguration("Log Creator Settings", panel).equals(InteractionResult.CONTINUE)) {

			LogCreatorParam param = new LogCreatorParam(mapping);
			param.setCreateNewEventClassesBasedOnMovements(panel.isCreateEventClassesBasedOnMovements());
			param.setCreateEventClassForUnmappedTransitions(panel.isCreateEventForUnmappedTransition());
			param.setCreateEventsForMoveLog(panel.isCreateEventForMoveLog());
			param.setCreateEventsForMoveModel(panel.isCreateEventForMoveModel());
			param.setMinFitnessValue(panel.getMinFitnessValue());
			param.setMaxFitnessValue(panel.getMaxFitnessValue());

			// create log with default param
			ILogCreatorAlgorithm algorithm = panel.getAlgorithm();
			XLog resLog = algorithm.extractLog(context, logReplayResult, log, param);

			if (resLog != null) {
				String inputValue = JOptionPane
						.showInputDialog("Input the name of the extracted log (empty for default)");
				if (inputValue.trim().isEmpty()) {
					context.getFutureResult(0).setLabel("Extracted log from alignment");
				} else {
					context.getFutureResult(0).setLabel(inputValue);
				}

				return resLog;
			}
			context.getFutureResult(0).cancel(true);
			return null;

		} else {
			context.getFutureResult(0).cancel(true);
			return null;
		}
	}
}
