/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.export;

import org.deckfour.xes.classification.XEventClassifier;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;

/**
 * @author aadrians
 * Jan 18, 2012
 *
 */
public class LogCreatorParam {
	/**
	 * Create new set of event classes based on move log/move model
	 * if set to false, only event classes from the original log is used
	 */
	private boolean createNewEventClassesBasedOnMovements = true;
	
	/**
	 * Create specific event class for transitions that are mapped to dummies
	 * if set to false, occurrence of invisible transitions would not appear
	 * as event in the log
	 */
	private boolean createEventClassForUnmappedTransitions = true;
	
	/**
	 * Occurrence of each move model only appears as an event 
	 */
	private boolean createEventsForMoveModel = true;
	
	/**
	 * Occurrence of each move log only appears as an event
	 */
	private boolean createEventsForMoveLog = true;
	
	/**
	 * Range of sync replay result that is going to be considered
	 */
	private double minFitnessValue = -1 * (Double.MAX_VALUE);
	private double maxFitnessValue = Double.MAX_VALUE; 
	
	// mapping from transition to event class, in case move on model occur
	private TransEvClassMapping mapping;
	
	@SuppressWarnings("unused")
	private LogCreatorParam(){}
	
	/**
	 * Default constructor
	 * @param mapping
	 */
	public LogCreatorParam(TransEvClassMapping mapping){
		this.mapping = mapping;
	}
	
	/**
	 * @return the createEventClassForUnmappedTransitions
	 */
	public boolean isCreateEventClassForUnmappedTransitions() {
		return createEventClassForUnmappedTransitions;
	}

	/**
	 * @param createEventClassForUnmappedTransitions the createEventClassForUnmappedTransitions to set
	 */
	public void setCreateEventClassForUnmappedTransitions(boolean createEventClassForUnmappedTransitions) {
		this.createEventClassForUnmappedTransitions = createEventClassForUnmappedTransitions;
	}

	/**
	 * @return the createEventsForMoveModel
	 */
	public boolean isCreateEventsForMoveModel() {
		return createEventsForMoveModel;
	}

	/**
	 * @param createEventsForMoveModel the createEventsForMoveModel to set
	 */
	public void setCreateEventsForMoveModel(boolean createEventsForMoveModel) {
		this.createEventsForMoveModel = createEventsForMoveModel;
	}

	/**
	 * @return the createEventsForMoveLog
	 */
	public boolean isCreateEventsForMoveLog() {
		return createEventsForMoveLog;
	}

	/**
	 * @param createEventsForMoveLog the createEventsForMoveLog to set
	 */
	public void setCreateEventsForMoveLog(boolean createEventsForMoveLog) {
		this.createEventsForMoveLog = createEventsForMoveLog;
	}

	/**
	 * @return the createNewEventClassesBasedOnMovements
	 */
	public boolean isCreateNewEventClassesBasedOnMovements() {
		return createNewEventClassesBasedOnMovements;
	}

	/**
	 * @param createNewEventClassesBasedOnMovements the createNewEventClassesBasedOnMovements to set
	 */
	public void setCreateNewEventClassesBasedOnMovements(boolean createNewEventClassesBasedOnMovements) {
		this.createNewEventClassesBasedOnMovements = createNewEventClassesBasedOnMovements;
	}

	/**
	 * @return the mapping
	 */
	public TransEvClassMapping getMapping() {
		return mapping;
	}

	/**
	 * @param mapping the mapping to set
	 */
	public void setMapping(TransEvClassMapping mapping) {
		this.mapping = mapping;
	}	
	
	/**
	 * get event classifier from the mapping
	 * @return
	 */
	public XEventClassifier getEventClassifier(){
		return mapping.getEventClassifier();
	}

	/**
	 * @return the minFitnessValue
	 */
	public double getMinFitnessValue() {
		return minFitnessValue;
	}

	/**
	 * @param minFitnessValue the minFitnessValue to set
	 */
	public void setMinFitnessValue(double minFitnessValue) {
		this.minFitnessValue = minFitnessValue;
	}

	/**
	 * @return the maxFitnessValue
	 */
	public double getMaxFitnessValue() {
		return maxFitnessValue;
	}

	/**
	 * @param maxFitnessValue the maxFitnessValue to set
	 */
	public void setMaxFitnessValue(double maxFitnessValue) {
		this.maxFitnessValue = maxFitnessValue;
	}
	
	
}
