/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.export.decisionmining;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.SortedSet;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.PNRepResultAllRequiredParamConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;

/**
 * @author aadrians Jun 26, 2013
 * 
 */
@Plugin(name = "Export Alignment to CSV for Deviation Analysis", returnLabels = {}, returnTypes = {}, parameterLabels = {
		"Petrinet", "Log", "Initial Marking", "Alignment" }, help = "This plugin export alignment into CSV file that can be imported using data mining tools (e.g. rapidminer) for decision point analysis.", userAccessible = true)
public class ExportAlignmentsToRapidminerCSV {
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Arya Adriansyah", email = "a.adriansyah@tue.nl", pack = "PNetAlignmentAnalysis")
	@PluginVariant(variantLabel = "From Petri net", requiredParameterLabels = { 0, 1, 3 })
	public void runExperiments(final UIPluginContext context, PetrinetGraph net, XLog log, PNRepResult repResult) {
		try {
			// get init marking from connection
			InitialMarkingConnection conn = context.getConnectionManager().getFirstConnection(
					InitialMarkingConnection.class, context, net);
			Marking initMarking = conn.getObjectWithRole(InitialMarkingConnection.MARKING);

			// populate all possible classifiers		
			PNRepResultAllRequiredParamConnection allParam = context.getConnectionManager().getFirstConnection(
					PNRepResultAllRequiredParamConnection.class, context, net, log, repResult);
			TransEvClassMapping trans2EvClassMapping = allParam
					.getObjectWithRole(PNRepResultAllRequiredParamConnection.TRANS2EVCLASSMAPPING);

			Set<XEventClassifier> classifiers = new HashSet<XEventClassifier>(log.getClassifiers());
			if ((log.getClassifiers() == null) || (log.getClassifiers().isEmpty())) {
				classifiers.add(XLogInfoImpl.STANDARD_CLASSIFIER);
				classifiers.add(XLogInfoImpl.NAME_CLASSIFIER);
				classifiers.add(XLogInfoImpl.RESOURCE_CLASSIFIER);
			}

			JFileChooser fc = new JFileChooser();
			if (fc.showSaveDialog(new JPanel()) == JFileChooser.APPROVE_OPTION) {
				ExportToRapidminerParam param = new ExportToRapidminerParam();
				param.setAlignment(repResult);
				param.setInitMarking(initMarking);
				param.setLog(log);
				param.setClassifiers(new XEventClassifier[] { trans2EvClassMapping.getEventClassifier() }); // always similar to the one used for replay
				param.setNet(net);
				param.setTargetPath(fc.getSelectedFile().getAbsolutePath());
				if (exportToCSVForMining(param)) {
					JOptionPane
							.showMessageDialog(new JPanel(),
									"CSV file is saved successfully. Use data mining tools (e.g. Rapidminer) to mine important decision factor.");
				} else {
					JOptionPane.showMessageDialog(new JPanel(),
							"Export fails. Check console output. Plugin terminates.");
				}
				;
			}
		} catch (Exception exc) {

		}
		context.getFutureResult(0).cancel(true);
		return;
	}

	@SuppressWarnings("unchecked")
	public boolean exportToCSVForMining(ExportToRapidminerParam param) {
		FileWriter fstream = null;
		BufferedWriter out = null;

		// init stat counter
		int[] vals = null; // move sync (trans) - move model (trans) - move log (all event classes) - all event classes without move log 
		String[] traceAttr = null; // trace attributes

		try {
			fstream = new FileWriter(param.getTargetPath());
			out = new BufferedWriter(fstream);

			// utilities
			XEventClasses[] eclasses = new XEventClasses[param.getClassifiers().length];
			int[] accumulativeClassNum = new int[eclasses.length];
			ArrayList<Map<XEventClass, Integer>> mapEclasses2Int = new ArrayList<Map<XEventClass, Integer>>(
					param.getClassifiers().length);
			int totalEvClass = 0;
			int counterNumValues = 0;

			// optional variables
			Map<Transition, Integer> trans2int = null; // mapping from transitions to integer
			Transition[] int2trans = null;

			Map<String, Integer> traceAtt2int = null; // mapping from trace attribute to integer
			String[] int2traceAtt = null;

			// encode everything 
			// transition
			trans2int = encodeTransitions(param.getNet(), out);
			int2trans = new Transition[trans2int.size()];
			for (Entry<Transition, Integer> e : trans2int.entrySet()) {
				int2trans[e.getValue()] = e.getKey();
			}
			counterNumValues += 2 * int2trans.length;

			// event classes for each classifiers
			int prevAccumulation = 0;
			String toBeWritten = "";
			for (int i = 0; i < param.getClassifiers().length; i++) {
				eclasses[i] = XLogInfoFactory.createLogInfo(param.getLog(), param.getClassifiers()[i])
						.getEventClasses();
				Object[] res = encodeEventClasses(param.getLog(), out, eclasses[i]);
				mapEclasses2Int.add((Map<XEventClass, Integer>) res[0]);
				toBeWritten += ((String) res[1]);
				accumulativeClassNum[i] = prevAccumulation;
				prevAccumulation += eclasses[i].size();
				totalEvClass += eclasses[i].size();
				counterNumValues += 2 * eclasses[i].size(); // move log and move synchronous
			}
			out.write(toBeWritten);

			// trace attributes
			traceAtt2int = new HashMap<String, Integer>(1); // collected while iterating the first trace

			// first trace to identify what case attributes exist
			XTrace t = param.getLog().get(0);
			int counter = 0;
			XAttributeMap allAttr = t.getAttributes();
			int2traceAtt = new String[allAttr.size()];
			for (Entry<String, XAttribute> e : allAttr.entrySet()) {
				traceAtt2int.put(e.getKey(), counter);
				int2traceAtt[counter] = e.getKey();

				out.write("\"attTrace:");
				out.write(e.getKey());
				out.write("\",");

				counter++;
			}

			out.newLine();

			// init stat counter
			vals = new int[counterNumValues];
			traceAttr = new String[int2traceAtt.length];

			// iterate through all traces based on alignment
			XLog log = param.getLog();

			for (SyncReplayResult syncRepRes : param.getAlignment()) {
				List<Object> nodeInstance = syncRepRes.getNodeInstance();
				List<StepTypes> steps = syncRepRes.getStepTypes();
				SortedSet<Integer> traceIndex = syncRepRes.getTraceIndex();

				for (Integer currTraceIdx : traceIndex) {
					XTrace currTrace = log.get(currTraceIdx);
					Iterator<XEvent> itEvt = currTrace.iterator();
					Iterator<Object> itNodeInstance = nodeInstance.iterator();
					Iterator<StepTypes> itSteps = steps.iterator();

					// reset stat counter
					Arrays.fill(vals, 0);

					// iterate events and record statistics until stopTrans is found
					Transition stopT = null;

					// collect all case attribute
					for (Entry<String, XAttribute> e : currTrace.getAttributes().entrySet()) {
						traceAttr[traceAtt2int.get(e.getKey())] = e.getValue().toString();
					}

					// move sync (trans) - move model (trans) - move log (all event classes) - all event classes without move log
					// timestamp
					while ((stopT == null) && (itSteps.hasNext())) {
						switch (itSteps.next()) {
							case LMGOOD :
								Transition trans = (Transition) itNodeInstance.next();
								XEvent event = itEvt.next();

								// collect all stats
								vals[trans2int.get(trans)]++; // move sync, transition is updated

								// update event classes
								for (int i = 0; i < eclasses.length; i++) {
									// for this classifier, get event class index and increase it
									int index = mapEclasses2Int.get(i).get(eclasses[i].getClassOf(event))
											+ accumulativeClassNum[i] + (int2trans.length * 2) + totalEvClass;
									vals[index]++;
								}
								break;
							case MINVI :
							case MREAL :
								Transition transMoveModel = (Transition) itNodeInstance.next();

								// collect stats
								vals[trans2int.get(transMoveModel) + int2trans.length]++; // move model

								break;
							case L :
								XEvent eventMoveLog = itEvt.next();
								itNodeInstance.next();

								// update event classes
								for (int i = 0; i < eclasses.length; i++) {
									// for this classifier, get event class index and increase it
									int index = mapEclasses2Int.get(i).get(eclasses[i].getClassOf(eventMoveLog))
											+ accumulativeClassNum[i] + (int2trans.length * 2);
									vals[index]++;
								}
								break;
							default :
								break;
						}
					}
					// write stats to file
					// transitions and event classes
					for (int i = 0; i < vals.length; i++) {
						out.write("\"");
						out.write(String.valueOf(vals[i]));
						out.write("\",");
					}

					// trace attributes
					for (String att : traceAttr) {
						out.write("\"");
						out.write(att);
						out.write("\",");
					}
					out.newLine();

				}
			}

			return true;
		} catch (Exception exc) {
			exc.printStackTrace();
			return false;
		} finally {
			if (out != null) {
				try {
					out.close();
				} catch (Exception exc3) {
					exc3.printStackTrace();
				}
			}
		}
	}

	private Object[] encodeEventClasses(XLog log, BufferedWriter out, XEventClasses eventClass) throws IOException {
		Map<XEventClass, Integer> ec2int = new HashMap<XEventClass, Integer>(eventClass.size());
		int counter = 0;
		String toBeWritten = "";
		for (XEventClass ec : eventClass.getClasses()) {
			ec2int.put(ec, counter);
			counter++;

			// write to output file
			out.write("\"mvLogEC:");
			out.write(ec.getId());
			out.write("\",");
			toBeWritten += "\"mvSyncEC:" + ec.getId() + "\",";

		}
		return new Object[] { ec2int, toBeWritten };
	}

	private Map<Transition, Integer> encodeTransitions(PetrinetGraph net, BufferedWriter out) throws IOException {
		Collection<Transition> colTrans = net.getTransitions();
		Map<Transition, Integer> trans2int = new HashMap<Transition, Integer>(colTrans.size());
		int counter = 0;
		String toBeWritten = "";
		for (Transition t : colTrans) {
			trans2int.put(t, counter);
			counter++;

			// write to output file
			out.write("\"mvSyncT:");
			out.write(t.getLabel());
			out.write("(");
			out.write(String.valueOf(counter));
			out.write(")");
			out.write("\",");
			toBeWritten += "\"mvModelT:";
			toBeWritten += t.getLabel();
			toBeWritten += "(";
			toBeWritten += String.valueOf(counter);
			toBeWritten += ")";
			toBeWritten += "\",";
		}
		out.write(toBeWritten);
		return trans2int;
	}
}
