/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.export.decisionmining;

import info.clearthought.layout.TableLayout;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.SortedSet;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.deckfour.uitopia.api.event.TaskListener;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.jfree.ui.FilesystemFilter;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.ui.widgets.ProMList;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.models.connections.petrinets.PNRepResultAllRequiredParamConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * Export aligment to CSV file that can be imported using data mining tools
 * (e.g. Rapidminer) for further analysis, such as Decision Point Analysis
 * 
 * @author aadrians Aug 9, 2012
 * 
 */
@Plugin(name = "Export Alignment to CSV for Decision Point Analysis", returnLabels = {}, returnTypes = {}, parameterLabels = {
		"Petrinet", "Log", "Initial Marking", "Alignment", "Stop Transitions", "Classifier" }, help = "This plugin export alignment into CSV file that can be imported using data mining tools (e.g. rapidminer) for decision point analysis.", userAccessible = true)
public class ExportToRapidminer {
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Arya Adriansyah", email = "a.adriansyah@tue.nl", pack = "PNetAlignmentAnalysis")
	@PluginVariant(variantLabel = "From Petri net", requiredParameterLabels = { 0, 1, 3 })
	public Object[] runExperiments(final UIPluginContext context, PetrinetGraph net, XLog log, PNRepResult repResult) {
		try {
			// get init marking from connection
			InitialMarkingConnection conn = context.getConnectionManager().getFirstConnection(
					InitialMarkingConnection.class, context, net);
			Marking initMarking = conn.getObjectWithRole(InitialMarkingConnection.MARKING);

			// populate all possible classifiers		
			PNRepResultAllRequiredParamConnection allParam = context.getConnectionManager().getFirstConnection(
					PNRepResultAllRequiredParamConnection.class, context, net, log, repResult);
			TransEvClassMapping trans2EvClassMapping = allParam
					.getObjectWithRole(PNRepResultAllRequiredParamConnection.TRANS2EVCLASSMAPPING);

			Set<XEventClassifier> classifiers = new HashSet<XEventClassifier>(log.getClassifiers());
			if ((log.getClassifiers() == null) || (log.getClassifiers().isEmpty())) {
				classifiers.add(XLogInfoImpl.STANDARD_CLASSIFIER);
				classifiers.add(XLogInfoImpl.NAME_CLASSIFIER);
				classifiers.add(XLogInfoImpl.RESOURCE_CLASSIFIER);
			}

			StopTransitionPanel panel = new StopTransitionPanel(net, classifiers,
					trans2EvClassMapping.getEventClassifier());
			panel.setPreferredSize(new Dimension(500, 200));
			if (context.showConfiguration("Export Settings for Rapidminer* tool", panel) == TaskListener.InteractionResult.CONTINUE) {
				if ((!panel.getSelectedFile().isEmpty()) && (panel.getStopTransitions().size() > 0)) {
					ExportToRapidminerParam param = new ExportToRapidminerParam();
					param.setAlignment(repResult);
					param.setInitMarking(initMarking);
					param.setLog(log);
					param.setNet(net);
					param.setSelectedDecisions(panel.getStopTransitions());
					param.setTargetPath(panel.getSelectedFile());
					param.setClassifiers(panel.getSelectedClassifiers());

					if (exportToCSVForMining(param)) {
						JOptionPane
								.showMessageDialog(new JPanel(),
										"CSV file is saved successfully. Use data mining tools (e.g. Rapidminer) to mine important decision factor.");
					}
					;
				} else {
					JOptionPane.showMessageDialog(new JPanel(),
							"No stop transitions are selected/proper file name inserted. Plugin terminates.");
				}
			}
			;

		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}

		context.getFutureResult(0).cancel(true);
		return null;
	}

	@SuppressWarnings("unchecked")
	public boolean exportToCSVForMining(ExportToRapidminerParam param) {
		FileWriter fstream = null;
		BufferedWriter out = null;

		// init stat counter
		int[] vals = null; // move sync (trans) - move model (trans) - move log (all event classes) - all event classes without move log 
		String[] traceAttr = null; // trace attributes

		try {
			fstream = new FileWriter(param.getTargetPath());
			out = new BufferedWriter(fstream);

			// utilities
			XEventClasses[] eclasses = new XEventClasses[param.getClassifiers().length];
			int[] accumulativeClassNum = new int[eclasses.length];
			ArrayList<Map<XEventClass, Integer>> mapEclasses2Int = new ArrayList<Map<XEventClass, Integer>>(
					param.getClassifiers().length);
			int totalEvClass = 0;
			int counterNumValues = 0;
			Date startTime = null;
			Date endTime = null;

			// optional variables
			Map<Transition, Integer> trans2int = null; // mapping from transitions to integer
			Transition[] int2trans = null;

			Map<String, Integer> traceAtt2int = null; // mapping from trace attribute to integer
			String[] int2traceAtt = null;

			// encode everything 
			// transition
			trans2int = encodeTransitions(param.getNet(), out);
			int2trans = new Transition[trans2int.size()];
			for (Entry<Transition, Integer> e : trans2int.entrySet()) {
				int2trans[e.getValue()] = e.getKey();
			}
			counterNumValues += 2 * int2trans.length;

			// event classes for each classifiers
			int prevAccumulation = 0;
			String toBeWritten = "";
			for (int i = 0; i < param.getClassifiers().length; i++) {
				eclasses[i] = XLogInfoFactory.createLogInfo(param.getLog(), param.getClassifiers()[i])
						.getEventClasses();
				Object[] res = encodeEventClasses(param.getLog(), out, eclasses[i]);
				mapEclasses2Int.add((Map<XEventClass, Integer>) res[0]);
				toBeWritten += ((String) res[1]);
				accumulativeClassNum[i] = prevAccumulation;
				prevAccumulation += eclasses[i].size();
				totalEvClass += eclasses[i].size();
				counterNumValues += 2 * eclasses[i].size(); // move log and move synchronous
			}
			out.write(toBeWritten);

			// time elapsed
			out.write("\"ELAPSED\",");

			// trace attributes
			traceAtt2int = new HashMap<String, Integer>(1); // collected while iterating the first trace

			// first trace to identify what case attributes exist
			XTrace t = param.getLog().get(0);
			int counter = 0;
			XAttributeMap allAttr = t.getAttributes();
			int2traceAtt = new String[allAttr.size()];
			for (Entry<String, XAttribute> e : allAttr.entrySet()) {
				traceAtt2int.put(e.getKey(), counter);
				int2traceAtt[counter] = e.getKey();

				out.write("\"attTrace:");
				out.write(e.getKey());
				out.write("\",");

				counter++;
			}

			out.write("\"DECISION\"");
			out.newLine();

			// init stat counter
			vals = new int[counterNumValues];
			traceAttr = new String[int2traceAtt.length];

			// iterate through all traces based on alignment
			XLog log = param.getLog();
			XTimeExtension te = XTimeExtension.instance();

			for (SyncReplayResult syncRepRes : param.getAlignment()) {
				List<Object> nodeInstance = syncRepRes.getNodeInstance();
				List<StepTypes> steps = syncRepRes.getStepTypes();
				SortedSet<Integer> traceIndex = syncRepRes.getTraceIndex();

				for (Integer currTraceIdx : traceIndex) {
					XTrace currTrace = log.get(currTraceIdx);
					Iterator<XEvent> itEvt = currTrace.iterator();
					Iterator<Object> itNodeInstance = nodeInstance.iterator();
					Iterator<StepTypes> itSteps = steps.iterator();

					// reset stat counter
					Arrays.fill(vals, 0);
					startTime = null;
					endTime = null;

					// iterate events and record statistics until stopTrans is found
					Transition stopT = null;

					// collect all case attribute
					for (Entry<String, XAttribute> e : currTrace.getAttributes().entrySet()) {
						traceAttr[traceAtt2int.get(e.getKey())] = e.getValue().toString();
					}

					// move sync (trans) - move model (trans) - move log (all event classes) - all event classes without move log
					// timestamp
					while ((stopT == null) && (itSteps.hasNext())) {
						switch (itSteps.next()) {
							case LMGOOD :
								Transition trans = (Transition) itNodeInstance.next();
								XEvent event = itEvt.next();

								if (startTime == null) { // timestamp
									startTime = te.extractTimestamp(event);
									endTime = startTime;
								} else {
									endTime = te.extractTimestamp(event);
								}

								if (param.getSelectedDecisions().contains(trans)) {
									stopT = trans;
								}
								// collect all stats
								vals[trans2int.get(trans)]++; // move sync, transition is updated

								// update event classes
								for (int i = 0; i < eclasses.length; i++) {
									// for this classifier, get event class index and increase it
									int index = mapEclasses2Int.get(i).get(eclasses[i].getClassOf(event))
											+ accumulativeClassNum[i] + (int2trans.length * 2) + totalEvClass;
									vals[index]++;
								}
								break;
							case MINVI :
							case MREAL :
								Transition transMoveModel = (Transition) itNodeInstance.next();

								if (param.getSelectedDecisions().contains(transMoveModel)) {
									stopT = transMoveModel;
								}

								// collect stats
								vals[trans2int.get(transMoveModel) + int2trans.length]++; // move model

								break;
							case L :
								XEvent eventMoveLog = itEvt.next();
								itNodeInstance.next();

								// update event classes
								for (int i = 0; i < eclasses.length; i++) {
									// for this classifier, get event class index and increase it
									int index = mapEclasses2Int.get(i).get(eclasses[i].getClassOf(eventMoveLog))
											+ accumulativeClassNum[i] + (int2trans.length * 2);
									vals[index]++;
								}
								break;
							default :
								break;
						}
					}
					if (stopT != null) {
						// write stats to file
						// transitions and event classes
						for (int i = 0; i < vals.length; i++) {
							out.write("\"");
							out.write(String.valueOf(vals[i]));
							out.write("\",");
						}

						// time elapsed
						out.write("\"");
						if ((endTime != null) && (startTime != null)) {
							out.write(String.valueOf(endTime.getTime() - startTime.getTime()));
						} else {
							out.write("0");
						}
						out.write("\",");

						// trace attributes
						for (String att : traceAttr) {
							out.write("\"");
							out.write(att);
							out.write("\",");
						}

						// label
						out.write("\"");
						out.write(stopT.getLabel());
						out.write("\"");

						out.newLine();
					}

				}
			}

			return true;
		} catch (Exception exc) {
			exc.printStackTrace();
			return false;
		} finally {
			if (out != null) {
				try {
					out.close();
				} catch (Exception exc3) {
					exc3.printStackTrace();
				}
			}
		}
	}

	private Object[] encodeEventClasses(XLog log, BufferedWriter out, XEventClasses eventClass) throws IOException {
		Map<XEventClass, Integer> ec2int = new HashMap<XEventClass, Integer>(eventClass.size());
		int counter = 0;
		String toBeWritten = "";
		for (XEventClass ec : eventClass.getClasses()) {
			ec2int.put(ec, counter);
			counter++;

			// write to output file
			out.write("\"mvLogEC:");
			out.write(ec.getId());
			out.write("\",");
			toBeWritten += "\"mvSyncEC:" + ec.getId() + "\",";

		}
		return new Object[] { ec2int, toBeWritten };
	}

	private Map<Transition, Integer> encodeTransitions(PetrinetGraph net, BufferedWriter out) throws IOException {
		Collection<Transition> colTrans = net.getTransitions();
		Map<Transition, Integer> trans2int = new HashMap<Transition, Integer>(colTrans.size());
		int counter = 0;
		String toBeWritten = "";
		for (Transition t : colTrans) {
			trans2int.put(t, counter);
			counter++;

			// write to output file
			out.write("\"mvSyncT:");
			out.write(t.getLabel());
			out.write("\",");
			toBeWritten += "\"mvModelT:";
			toBeWritten += t.getLabel();
			toBeWritten += "\",";
		}
		out.write(toBeWritten);
		return trans2int;
	}

}

class StopTransitionPanel extends JPanel {
	private static final long serialVersionUID = 1L;
	private ProMList list;
	private DefaultListModel listModel;
	private ProMTextField tf;

	// classifiers selection
	private ProMList listC;
	private DefaultListModel listModelC;

	public StopTransitionPanel(PetrinetGraph net, Set<XEventClassifier> classifiers, XEventClassifier defaultSelection) {
		// stop transitions
		List<Transition> listTrans = new ArrayList<Transition>(net.getTransitions());
		Collections.sort(listTrans, new Comparator<Transition>() {

			public int compare(Transition o1, Transition o2) {
				return o1.getLabel().compareTo(o2.getLabel());
			}
		});

		listModel = new DefaultListModel();
		for (Transition t : listTrans) {
			listModel.addElement(t);
		}
		list = new ProMList("Select transitions that represent decisions**", listModel);

		// button to select file
		tf = new ProMTextField(System.getProperty("user.home") + "\\alignment.csv");
		tf.setEditable(false);

		SlickerFactory factory = SlickerFactory.instance();
		JButton button = factory.createButton("Change target file...");
		button.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				final JFileChooser fc = new JFileChooser();
				fc.setFileFilter(new FilesystemFilter(".CSV", "Comma-separated values (.CSV)"));
				if (JFileChooser.APPROVE_OPTION == fc.showSaveDialog(new JPanel())) {
					if (fc.getSelectedFile().getAbsolutePath().toLowerCase().endsWith(".csv")) {
						tf.setText(fc.getSelectedFile().getAbsolutePath());
					} else {
						tf.setText(fc.getSelectedFile().getAbsolutePath() + ".csv");
					}
				}
				;
			}
		});

		// classifiers
		listModelC = new DefaultListModel();
		for (XEventClassifier clazz : classifiers) {
			listModelC.addElement(clazz);
		}
		listC = new ProMList("Select classifiers to extract item information from events**", listModelC);
		if (listModelC.contains(defaultSelection)) {
			listC.setSelection(defaultSelection);
		}

		// put them in the panel
		double size[][] = new double[][] { { 500, 200 }, { 220, 1, 125, 20, 20, 30 } };
		setLayout(new TableLayout(size));

		add(list, "0, 0, 1, 0");
		add(listC, "0,2,1,2");
		add(factory
				.createLabel("<html>* Rapidminer can be downloaded from <a href=\"http://rapid-i.com/\">http://rapid-i.com/</a></html>"),
				"0,3,1,3");
		add(factory.createLabel("** Hold CTRL button while clicking selection to choose more than 1 items"), "0,4,1,4");
		add(tf, "0, 5");
		add(button, "1, 5");

		setPreferredSize(new Dimension(400, 515));
		setMinimumSize(new Dimension(400, 515));
	}

	public XEventClassifier[] getSelectedClassifiers() {
		Object[] selectedVals = listC.getSelectedValues();
		XEventClassifier[] res = new XEventClassifier[selectedVals.length];
		for (int i = 0; i < selectedVals.length; i++) {
			res[i] = (XEventClassifier) selectedVals[i];
		}
		return res;
	}

	public String getSelectedFile() {
		return tf.getText();
	}

	public Set<Transition> getStopTransitions() {
		Object[] vals = list.getSelectedValues();
		Set<Transition> stopTransitions = new HashSet<Transition>();

		for (Object val : vals) {
			stopTransitions.add((Transition) val);
		}
		return stopTransitions;
	}
}
