package org.processmining.plugins.pnalignanalysis.visualization.projection;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JPanel;

import org.processmining.plugins.petrinet.visualization.AlignmentConstants;

public class LegendConfSpecific extends JPanel {
	private static final long serialVersionUID = -1668066830588872184L;

	private static final Font boldFont = new Font("Sans", Font.BOLD, 12);
	private static final Font normalFont = new Font("Sans", Font.PLAIN, 12);
	private static final Font bigFont = new Font("Sans", Font.PLAIN, 24);
	private static final Font bigFontBold = new Font("Sans", Font.BOLD, 12);

	public LegendConfSpecific() {
		super();
	}

	public void paint(Graphics graphics) {
		Graphics2D g = (Graphics2D) graphics;

		// draw legend
		int padding = 8;
		int currX = padding;
		int currY = padding;
		FontMetrics fm = graphics.getFontMetrics(boldFont);
		FontMetrics fmBig = graphics.getFontMetrics(bigFont);
		FontMetrics fmBigBold = graphics.getFontMetrics(bigFontBold);
		int fontHeight = fm.getHeight();

		// draw box
		int lengthSync = 130;
		int lengthBox = 225;
		int heightBox = 80;

		g.setPaint(PNLogReplayProjectedVisPanel.MED);
		g.fillRect(currX, currY, lengthBox, heightBox);

		currX = padding + 20;
		currY += fmBig.getHeight();
		g.setColor(Color.BLACK);
		g.setFont(bigFont);
		g.drawString("Transition label", currX, currY);

		g.setFont(bigFontBold);
		currY += fmBigBold.getHeight() + 5;
		g.drawString("(Freq Sync/Freq move on model)", currX, currY);

		currX = padding;
		g.setPaint(AlignmentConstants.MOVESYNCCOLOR);
		g.fillRect(currX, (int) (padding + (0.9 * heightBox)), lengthSync, (int) (0.1 * heightBox));

		currX += lengthSync;
		g.setPaint(AlignmentConstants.MOVEMODELREALCOLOR);
		g.fillRect(currX, (int) (padding + (0.9 * heightBox)), lengthBox - lengthSync, (int) (0.1 * heightBox));

		currX = padding;
		currY = padding + heightBox + fontHeight;

		g.setPaint(Color.BLACK);
		g.setFont(normalFont);
		g.drawString("Synchronous moves", currX, currY);
		g.drawString("(ratio)", currX, currY + fontHeight);
		currX += fm.stringWidth("Synchronous moves") + 20;

		g.drawString("Move on model", currX, currY);
		g.drawString("(ratio)", fm.stringWidth("Move on model") + currX - fm.stringWidth("(ratio)"), currY +  fontHeight);

		currY += 30 + fontHeight;
		g.setFont(normalFont);
		g.drawString("- Transition color shows frequency",
				padding, currY);
		currY += fontHeight;
		g.drawString("- Red border means move on model > 0",
				padding, currY);
		currY += fontHeight;
		g.drawString("- Invisible transitions are colored grey",
				padding, currY);
		currY += fontHeight;
		
		g.setPaint(Color.RED);
		g.drawRect(padding, padding, lengthBox, heightBox);

		// place index
		g.setPaint(Color.YELLOW);
		g.fillOval(padding, currY, 10, 10);
		g.setPaint(Color.BLACK);
		g.drawOval(padding, currY, 10, 10);
		currY += 10;
		g.drawString("Marked place where move log occurred", padding + 15, currY);
		currY += 10;
		g.setPaint(Color.RED);
		g.fillOval(padding, currY, 10, 10);
		g.setPaint(Color.BLACK);
		g.drawOval(padding, currY, 10, 10);
		currY += 10;
		g.drawString("Other place marked when move log", padding + 15, currY);
		currY += fontHeight;
		g.drawString("occurred on selected place", padding + 15, currY);
		currY += 10;
		g.setPaint(Color.WHITE);
		g.fillOval(padding, currY, 10, 10);
		g.setPaint(Color.BLACK);
		g.drawOval(padding, currY, 10, 10);
		currY += 10;
		g.drawString("States where NO move log occurred", padding + 15, currY);
		currY += 10 + fontHeight;
		g.drawString("- Place size shows move on log frequency", padding, currY);
	}
}
