/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.visualization.projection;

import javax.swing.JComponent;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.PNRepResultAllRequiredParamConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;

/**
 * @author aadrians
 * Oct 26, 2011
 *
 */
@Plugin(name = "Model Projected with Alignments", level = PluginLevel.PeerReviewed, returnLabels = { "Projected Log-Model Alignment to Model" }, returnTypes = { JComponent.class }, parameterLabels = { "Log-Model alignment" }, userAccessible = true)
@Visualizer
public class PNLogReplayProjectedVis {
	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context, PNRepResult logReplayResult) throws ConnectionCannotBeObtained {
		System.gc();
		PetrinetGraph net = null;
		XLog log = null;
		TransEvClassMapping map = null;
		Marking initMarking = null;
		try {
			PNRepResultAllRequiredParamConnection conn = context.getConnectionManager().getFirstConnection(PNRepResultAllRequiredParamConnection.class, context, logReplayResult);
			
			net = conn.getObjectWithRole(PNRepResultAllRequiredParamConnection.PN);
			log = conn.getObjectWithRole(PNRepResultAllRequiredParamConnection.LOG);
			map = conn.getObjectWithRole(PNRepResultAllRequiredParamConnection.TRANS2EVCLASSMAPPING);
			
		} catch (Exception exc){
			context.log("No net can be found for this log replay result");
			context.getFutureResult(0).cancel(true);
			return null;
		}
		
		try {
			// get initial marking of the net
			InitialMarkingConnection initMarkingConn = context.getConnectionManager().getFirstConnection(InitialMarkingConnection.class, context, net);
			initMarking = initMarkingConn.getObjectWithRole(InitialMarkingConnection.MARKING);
		} catch (Exception exc){
			// it's okay not to have any initial marking
			initMarking = new Marking();
		}
		return new PNLogReplayProjectedVisPanel(context, net, initMarking, log, map, logReplayResult);
	}
}
