/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.visualization.tracealignment;

import java.awt.Color;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedSet;

import javax.swing.JComponent;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.PNRepResultAllRequiredParamConnection;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.petrinet.visualization.AlignmentConstants;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;

/**
 * 
 * @author aadrians
 * @mail a.adriansyah@tue.nl
 * @since Nov 10, 2011
 */
@Plugin(name = "Trace Alignment of Alignments", level = PluginLevel.PeerReviewed, returnLabels = { "Trace Alignment of Alignments" }, returnTypes = { JComponent.class }, parameterLabels = { "Log-Model alignment" }, userAccessible = true)
@Visualizer
public class PNLogReplayAlignmentVis {
	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(final PluginContext context, PNRepResult logReplayResult) {
		System.gc();
		try {
			// temp variables
			ActivityColorMap activityColorMap = new ActivityColorMap();

			// find previous log
			PNRepResultAllRequiredParamConnection conn = context.getConnectionManager().getFirstConnection(
					PNRepResultAllRequiredParamConnection.class, context, logReplayResult);
			XLog log = conn.getObjectWithRole(PNRepResultAllRequiredParamConnection.LOG);

			// create a log based on alignment
			XFactory factory = XFactoryRegistry.instance().currentDefault();
			XConceptExtension conceptExtension = XConceptExtension.instance();

			// result
			XLog outputLog = factory.createLog(log.getAttributes());

			// create event class for each pair of movement + log 
			for (SyncReplayResult repRes : logReplayResult) {
				// store all result
				SortedSet<Integer> traceIndexes = repRes.getTraceIndex();
				XTrace[] result = new XTrace[traceIndexes.size()];
				int ctr = 0;
				for (int index : traceIndexes) {
					result[ctr] = factory.createTrace(log.get(index).getAttributes());
					ctr++;
				}

				Iterator<Object> it = repRes.getNodeInstance().iterator();
				for (StepTypes stepType : repRes.getStepTypes()) {
					switch (stepType) {
						case L :
							addNewEvent(factory, conceptExtension, result, it.next(), AlignmentConstants.MOVELOG,
									AlignmentConstants.MOVELOGCOLOR, activityColorMap);
							break;
						case LMGOOD :
							addNewEvent(factory, conceptExtension, result, it.next(), AlignmentConstants.MOVESYNC, AlignmentConstants.MOVESYNCCOLOR,
									activityColorMap);
							break;
						case MINVI :
							addNewEvent(factory, conceptExtension, result, it.next(), AlignmentConstants.MOVEMODELINVI, AlignmentConstants.MOVEMODELINVICOLOR,
									activityColorMap);
							break;
						case MREAL :
							addNewEvent(factory, conceptExtension, result, it.next(), AlignmentConstants.MOVEMODELREAL,
									AlignmentConstants.MOVEMODELREALCOLOR, activityColorMap);
							break;
						case LMNOGOOD :
							addNewEvent(factory, conceptExtension, result, it.next(), AlignmentConstants.MOVESYNCVIOL, AlignmentConstants.MOVESYNCVIOLCOLOR,
									activityColorMap);
							break;
						case LMREPLACED:
							addNewEvent(factory, conceptExtension, result, it.next(), AlignmentConstants.MOVEREPLACED, AlignmentConstants.MOVEREPLACEDCOLOR,
									activityColorMap);
							break;
						case LMSWAPPED:
							addNewEvent(factory, conceptExtension, result, it.next(), AlignmentConstants.MOVESWAPPED, AlignmentConstants.MOVESWAPPEDCOLOR,
									activityColorMap);
							break;
					}
				}

				int i = 0;
				for (Integer traceIndex : repRes.getTraceIndex()) {
					XAttribute traceNameAttr = factory.createAttributeLiteral("concept:name",
							conceptExtension.extractName(log.get(traceIndex)), conceptExtension);
					result[i].getAttributes().put("concept:name", traceNameAttr);
					outputLog.add(result[i]);
					i++;
				}
			}

			// call trace alignment
			return TraceAlignmentDelegate.createAlignmentTree(context, outputLog, activityColorMap);			
		} catch (ConnectionCannotBeObtained connExc) {
			context.log("No net can be found for this log replay result");
			context.getFutureResult(0).cancel(true);
			return null;
		} catch (Exception exc) {
			context.log("Exception occur, see log");
			context.log(exc.getMessage());
			exc.printStackTrace();
			context.getFutureResult(0).cancel(true);
			return null;
		}
	}

	/**
	 * @param factory
	 * @param conceptExtension
	 * @param result
	 * @param it
	 */
	private void addNewEvent(XFactory factory, XConceptExtension conceptExtension, XTrace[] result, Object obj,
			String sign, Color color, Map<String, Color> activityColorMap) {
		activityColorMap.put(obj.toString() + sign + "-" + AlignmentConstants.LIFECYCLETRANSSTR, color);

		for (int i = 0; i < result.length; i++) {
			XEvent outputEvent = factory.createEvent();
			XAttribute name = factory.createAttributeLiteral("concept:name", obj.toString() + sign, conceptExtension);
			outputEvent.getAttributes().put("concept:name", name);
			XAttribute transition = factory.createAttributeLiteral("lifecycle:transition", AlignmentConstants.LIFECYCLETRANSSTR,
					XConceptExtension.instance());
			outputEvent.getAttributes().put("lifecycle:transition", transition);
			result[i].add(outputEvent);
		}
	}
	
}
